package org.jboss.soa.qa.hl7binging;

import org.apache.camel.CamelContext;
import org.apache.camel.ProducerTemplate;
import org.apache.camel.component.hl7.HL7MLLPCodec;
import org.apache.camel.impl.DefaultCamelContext;
import org.apache.camel.impl.SimpleRegistry;
import org.apache.mina.filter.codec.ProtocolCodecFactory;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.RunAsClient;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.JavaArchive;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.File;

import ca.uhn.hl7v2.DefaultHapiContext;
import ca.uhn.hl7v2.HapiContext;
import ca.uhn.hl7v2.model.Message;
import ca.uhn.hl7v2.parser.Parser;
import ca.uhn.hl7v2.validation.impl.NoValidation;

/**
 * Tests of HL7 binding to switchyard bean service.
 */
@RunWith(Arquillian.class)
public class Hl7ServiceBindingTest {

	private static final int MINA2_HL7_PORT = 8882;
	private static final int MINA2_HL7_NULL_RESPONSE_PORT = 8883;
	private static final String CODEC = "hl7codec";



	@Deployment(order = 1, managed = true, testable = true)
	public static JavaArchive createDeployment() {
		final String projectName = "hl7-nullsync-test";
		return ShrinkWrap
				.create(ZipImporter.class, projectName + ".jar")
				.importFrom(
						new File("target" + File.separator + projectName
								+ ".jar")).as(JavaArchive.class);
	}

	@BeforeClass
	public static void prepareClass() throws Exception {
		//wait for deploy and start bundle
		Thread.sleep(5000L);
	}

	/**
	 * HL7 binding sync test.
	 */
	@Test
	@RunAsClient
	public void testHl7ServiceSync() throws Exception {
		final Message requestObj = createMessageObj();
		final Message responseObj = sendRequest(requestObj, MINA2_HL7_PORT, true);
		assertMessageEquals(requestObj, responseObj);
	}

	/**
	 * HL7 binding async test.
	 */
	@Test
	@RunAsClient
	public void testHl7ServiceAsync() throws Exception {
		final Message requestObj = createMessageObj();
		final Message responseObj = sendRequest(requestObj, MINA2_HL7_PORT, false);
		Assert.assertNull(responseObj);
	}

	/**
	 * HL7 binding sync test with null reponse (throws exception).
	 */
	@Test
	@RunAsClient
	public void testHl7ServiceSyncMessageNullResponse() throws Exception {
		final Message requestObj = createMessageObj();
		//Here is the problem https://bugzilla.redhat.com/show_bug.cgi?id=1148729
		final Message responseObj = sendRequest(requestObj, MINA2_HL7_NULL_RESPONSE_PORT, true);
		Assert.assertNull(responseObj);
	}

	/**
	 * HL7 binding async test with null reponse.
	 */
	@Test
	@RunAsClient
	public void testHl7ServiceAsyncMessageNullResponse() throws Exception {
		final Message requestObj = createMessageObj();
		final Message responseObj = sendRequest(requestObj, MINA2_HL7_NULL_RESPONSE_PORT, false);
		Assert.assertNull(responseObj);
	}

	private static Message sendRequest(final Message request, final int port, final boolean sync) throws Exception {

		final SimpleRegistry registry = new SimpleRegistry();
		registry.put(CODEC, createHl7Codec());

		final CamelContext camelContext = new DefaultCamelContext(registry);
		camelContext.start();
		final ProducerTemplate template = camelContext.createProducerTemplate();
		final String url = MinaUrlBuilder.build(port, sync, CODEC);

		printSendingRequest(url, request);
		final Message response = template.requestBody(url, request, Message.class);
		printReceivedReponse(response);

		template.stop();
		camelContext.stop();

		return response;
	}

	private static void printSendingRequest(final String url, final Message request) {
		System.out.println("Sending request (to " + url + "): " + request.toString().replace("\r", "\n"));
	}

	private static void printReceivedReponse(final Message response) {
		if (response != null) {
			System.out.println("Received response: " + response.toString().replace("\r", "\n"));
		} else {
			System.out.println("None response received.");
		}
	}

	private static void assertMessageEquals(final Message expected, final Message actual) throws Exception {
		final String expectedNorm = getNormalizedHumanReadableMessage(expected);
		final String actualNorm = getNormalizedHumanReadableMessage(actual);
		Assert.assertEquals(expectedNorm, actualNorm);
	}

	private static String getNormalizedHumanReadableMessage(final Message message) throws Exception {
		final Parser parser = getNoValidatingParser();
		return parser.parse(message.toString()).toString().replace("\r", "\n");
	}

	private static ProtocolCodecFactory createHl7Codec() {
		final HL7MLLPCodec codec = new HL7MLLPCodec();
		codec.setValidate(false);
		codec.setCharset("iso-8859-1");
		return codec;
	}

	private static String createMessageStr() {
		final String line1 = "MSH|^~\\&|MYSENDER|MYRECEIVER|MYAPPLICATION||200612211200||QRY^A19|1224|P|2.4";
		final String line2 = "QRD|200612211200|R|I|GetPatient|||1^RD|0101701234|DEM||";
		return line1 + "\r" + line2; //all segments must be terminated by a CR
	}

	private static Message createMessageObj() throws Exception {
		final Parser parser = getNoValidatingParser();
		return parser.parse(createMessageStr());
	}

	private static Parser getNoValidatingParser() {
		final HapiContext hapiContext = new DefaultHapiContext();
		hapiContext.setValidationContext(new NoValidation());
		return hapiContext.getPipeParser();
	}



}
