package org.jboss.soa.qa.hl7binging;

import org.apache.camel.CamelContext;
import org.apache.camel.ProducerTemplate;
import org.apache.camel.impl.DefaultCamelContext;
import org.apache.camel.impl.SimpleRegistry;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.RunAsClient;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.osgi.StartLevelAware;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.importer.ZipImporter;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.soa.qa.hl7binding.echo.EchoServiceBean;
import org.jboss.soa.qa.hl7binding.transform.SerializationCodec;

import org.junit.Assert;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.junit.runner.RunWith;

import java.io.File;
import java.util.UUID;

/**
 * Tests for MINA2 protocol (without hl7) using serialization codec.
 */
@RunWith(Arquillian.class)
public class MinaEchoServiceTest {

	private static final String CODEC = "serializationCodec";
	private static final String MESSAGE = "Hello, this is test message for mina ping service.";

	private static final int MINA2_ECHO_PORT = 8880;
	private static final int MINA2_ECHO_NULL_RESPONSE_PORT = 8881;

	@Deployment(order = 1, managed = true, testable = true)
	public static JavaArchive createDeployment() {
		final String projectName = "hl7-nullsync-test";
		return ShrinkWrap
				.create(ZipImporter.class, projectName + ".jar")
				.importFrom(
						new File("target" + File.separator + projectName
								+ ".jar")).as(JavaArchive.class);
	}

	@BeforeClass
	public static void prepareClass() throws Exception {
		//wait for deploy and start bundle
		Thread.sleep(5000L);
	}

	/**
	 * Mina2 sync test.
	 */
	@Test
	@RunAsClient
	public void echoSyncTest() throws Exception {
		final String request = createRequest();
		final String response = sendRequest(request, MINA2_ECHO_PORT, true);
		Assert.assertEquals("REPLY:" + request, response);
	}

	/**
	 * Mina2 async test.
	 */
	@Test
	@RunAsClient
	public void echoAsyncTest() throws Exception {
		final String request = createRequest();
		final String response = sendRequest(request, MINA2_ECHO_PORT, false);
		Assert.assertNull(response);
	}

	/**
	 * Mina2 sync test with null response (throws exception).
	 */
	@Test
	@RunAsClient
	public void echoSyncNullResponseTest() throws Exception {
		final String request = createRequest();
		//Here is the problem https://bugzilla.redhat.com/show_bug.cgi?id=1148729
		final String response = sendRequest(request, MINA2_ECHO_NULL_RESPONSE_PORT, true);
		Assert.assertNull(response);
	}

	/**
	 * Mina2 async test with null response.
	 */
	@Test
	@RunAsClient
	public void echoAsyncNullResponseTest() throws Exception {
		final String request = createRequest();
		final String response = sendRequest(request, MINA2_ECHO_NULL_RESPONSE_PORT, false);
		Assert.assertNull(response);
	}

	private static String sendRequest(final String request, final int port, final boolean sync) throws Exception {
		final SimpleRegistry registry = new SimpleRegistry();
		final SerializationCodec serializationCodec = new SerializationCodec();
		registry.put(CODEC, serializationCodec);

		final CamelContext camelContext = new DefaultCamelContext(registry);
		camelContext.start();
		final ProducerTemplate template = camelContext.createProducerTemplate();
		final String url = MinaUrlBuilder.build(port, sync, CODEC);

		System.out.println("Sending request (to " + url + "): " + request);
		final String response = (String)template.requestBody(url, request);
		System.out.println("Received response: " + response);

		template.stop();
		camelContext.stop();

		return response;
	}

	private static String createRequest() {
		return MESSAGE + " " + UUID.randomUUID().toString();
	}

}
